@@ -13,8 +13,9 @@ import (
 const esc = '\x1b'
 
 type styledStringSplitter struct {
-	input      string
-	lineNumber *linenumbers.LineNumber
+	input          string
+	lineNumber     *linenumbers.LineNumber
+	plainTextStyle twin.Style
 
 	nextByteIndex     int
 	previousByteIndex int
@@ -29,17 +30,20 @@ type styledStringSplitter struct {
 }
 
 // Returns the style of the line's trailer
-func styledStringsFromString(s string, lineNumber *linenumbers.LineNumber, callback func(string, twin.Style)) twin.Style {
+func styledStringsFromString(plainTextStyle twin.Style, s string, lineNumber *linenumbers.LineNumber, callback func(string, twin.Style)) twin.Style {
 	if !strings.ContainsAny(s, "\x1b") {
 		// This shortcut makes BenchmarkPlainTextSearch() perform a lot better
-		callback(s, twin.StyleDefault)
-		return twin.StyleDefault
+		callback(s, plainTextStyle)
+		return plainTextStyle
 	}
 
 	splitter := styledStringSplitter{
-		input:      s,
-		lineNumber: lineNumber,
-		callback:   callback,
+		input:           s,
+		lineNumber:      lineNumber,
+		plainTextStyle:  plainTextStyle, // How plain text should be styled
+		inProgressStyle: plainTextStyle, // Plain text style until something else comes along
+		callback:        callback,
+		trailer:         plainTextStyle, // Plain text style until something else comes along
 	}
 	splitter.run()
 
@@ -117,6 +121,10 @@ func (s *styledStringSplitter) handleEscape() error {
 		// Got the start of a CSI or an OSC sequence
 		return s.consumeControlSequence(char)
 	}
+	if char == '(' {
+		// Designate G0 charset: https://www.xfree86.org/4.8.0/ctlseqs.html
+		return s.consumeG0Charset()
+	}
 
 	return fmt.Errorf("Unhandled Fe sequence ESC%c", char)
 }
@@ -151,6 +159,18 @@ func (s *styledStringSplitter) consumeControlSequence(charAfterEsc rune) error {
 	}
 }
 
+func (s *styledStringSplitter) consumeG0Charset() error {
+	// First char after "ESC("
+	char := s.nextChar()
+	if char == 'B' {
+		// G0 charset is now "B" (ASCII)
+		s.startNewPart(s.plainTextStyle)
+		return nil
+	}
+
+	return fmt.Errorf("Unhandled G0 charset: %c", char)
+}
+
 // If the whole CSI sequence is ESC[33m, you should call this function with just
 // "33m".
 func (s *styledStringSplitter) handleCompleteControlSequence(charAfterEsc rune, sequence string) error {
@@ -206,15 +226,79 @@ func (s *styledStringSplitter) handleOsc(sequence string) error {
 	return fmt.Errorf("Unhandled OSC sequence")
 }
 
+// Based on https://infra.spec.whatwg.org/#surrogate
+func isSurrogate(char rune) bool {
+	if char >= 0xD800 && char <= 0xDFFF {
+		return true
+	}
+
+	return false
+}
+
+// Non-characters end with 0xfffe or 0xffff, or are in the range 0xFDD0 to 0xFDEF.
+//
+// Based on https://infra.spec.whatwg.org/#noncharacter
+func isNonCharacter(char rune) bool {
+	if char >= 0xFDD0 && char <= 0xFDEF {
+		return true
+	}
+
+	// Non-characters end with 0xfffe or 0xffff
+	if char&0xFFFF == 0xFFFE || char&0xFFFF == 0xFFFF {
+		return true
+	}
+
+	return false
+}
+
+// Based on https://url.spec.whatwg.org/#url-code-points
+func isValidURLChar(char rune) bool {
+	if char == '\\' {
+		// Ref: https://github.com/walles/moar/issues/244#issuecomment-2350908401
+		return true
+	}
+
+	if char == '%' {
+		// Needed for % escapes
+		return true
+	}
+
+	// ASCII alphanumerics
+	if char >= '0' && char <= '9' {
+		return true
+	}
+	if char >= 'A' && char <= 'Z' {
+		return true
+	}
+	if char >= 'a' && char <= 'z' {
+		return true
+	}
+
+	if strings.ContainsRune("!$&'()*+,-./:;=?@_~", char) {
+		return true
+	}
+
+	if char < 0x00a0 {
+		return false
+	}
+
+	if char > 0x10FFFD {
+		return false
+	}
+
+	if isSurrogate(char) {
+		return false
+	}
+
+	if isNonCharacter(char) {
+		return false
+	}
+
+	return true
+}
+
 // We just got ESC]8; and should now read the URL. URLs end with ASCII 7 BEL or ESC \.
 func (s *styledStringSplitter) handleURL() error {
-	// Ref: https://github.com/walles/moar/issues/244#issuecomment-2350908401
-	const windowsURLChars = `\`
-
-	// Valid URL characters.
-	// Ref: https://stackoverflow.com/a/1547940/473672
-	const validChars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-._~:/?#[]@!$&'()*+,;=" + windowsURLChars
-
 	// Points to right after "ESC]8;"
 	urlStartIndex := s.nextByteIndex
 
@@ -252,8 +336,8 @@ func (s *styledStringSplitter) handleURL() error {
 			return nil
 		}
 
-		if !strings.ContainsRune(validChars, char) {
-			return fmt.Errorf("Invalid URL character: %q", char)
+		if !isValidURLChar(char) {
+			return fmt.Errorf("Invalid URL character: <%q>", char)
 		}
 
 		// It's a valid URL char, keep going
@@ -261,6 +345,10 @@ func (s *styledStringSplitter) handleURL() error {
 }
 
 func (s *styledStringSplitter) startNewPart(style twin.Style) {
+	if style == twin.StyleDefault {
+		style = s.plainTextStyle
+	}
+
 	if style == s.inProgressStyle {
 		// No need to start a new part
 		return
