@@ -10,14 +10,14 @@ import (
 func TestParseScrollHint(t *testing.T) {
 	token, err := parseScrollHint("ESC[7m>")
 	assert.NilError(t, err)
-	assert.Equal(t, token, twin.Cell{
+	assert.Equal(t, token, twin.StyledRune{
 		Rune:  '>',
 		Style: twin.StyleDefault.WithAttr(twin.AttrReverse),
 	})
 }
 
 func TestPageOneInputFile(t *testing.T) {
-	pager, screen, _, formatter, err := pagerFromArgs(
+	pager, screen, _, formatter, _, err := pagerFromArgs(
 		[]string{"", "moar_test.go"},
 		func(_ twin.MouseMode, _ twin.ColorCount) (twin.Screen, error) {
 			return twin.NewFakeScreen(80, 24), nil
